
% We repeat Kim and Oka's Table 3 by fixing their coding errors.


clear all;
addpath('./..');    %add directory that contains data 
addpath('./../..'); %add directory that contains function "LS_factor" 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%  LOAD DATA (data and code for loading are from Kim and Oka):
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

DATA  = csvread('DivorceRate1956_1988.csv', 1, 1);
% In csvread(), 2nd and 3rd arguments are starting row and col to read.

% Data Definition

% State ID and Year
% Note: DATA is sorted by id_st and year in order.
id_st = DATA(:,1);

% Drop IN (id == 16) and NM (id == 33)
ind02  = (id_st ~= 16 & id_st ~= 33);
DATA02 = DATA(ind02,:);
id_st = DATA02(:,1);
year  = DATA02(:,2);
Y02 = (DATA02(:,4)); % Dependent variable%

% State population:
stpop = DATA02(:,5);

% Dummy for Friedberg's unilateral law
d_unilateral = DATA02(:,6);

% Dynamic Treatment effects
% - - - Friedberg's unilateral law
dyn_unilateral = DATA02(:,7:14);

% sample size
NT = size(DATA02, 1);
T  = max(year) - min(year) + 1;
N  = NT / T;

% dimension of the regressors
K = 8;   %will add +1 for lagged dependent variable below

% dependend variable and regressors in NT-vector form:

YY=Y02;
XX=dyn_unilateral;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%  TRANSFORM DATA INTO NxT MATRIX FORM:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Y = (reshape(YY,[T,N]))';
X = zeros(K,N,T);
for k=1:K
  X(k,:,:) = (reshape(XX(:,k),[T,N]))';
end

K
N
T


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%  ESTIMATION:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

beta0 = zeros(K,1);    
Rmax = 9 % the maximum number of the factors

lambda_known=ones(N,1);  % control for standard time effects
% f_known=ones(T,1);       % control for standard individual specific effects


trend=(1:T)';
% f_fnown=[ones(T,1),trend]; %control for standard individual specific effects + time trend
f_known=[ones(T,1),trend,trend.^2]; %control for standard individual specific effects + time trend + time trend.^2

for R = 0:Rmax
  [beta,exitflag,lambda,f,Vbeta1,Vbeta2,Vbeta3,bcorr1,bcorr2,bcorr3]=LS_factor2(Y,X,R,lambda_known,f_known,'silent',10^-8,'m1',beta0,30,300,2,2);
  
  beta_naiv(:,R+1)=beta;
  beta_bias_corrected(:,R+1)=beta - bcorr2 - bcorr3;
  std_est_homo(:,R+1)=sqrt(diag(Vbeta1));
  std_est_hetero(:,R+1)=sqrt(diag(Vbeta2));
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%  REPORT RESULTS:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

est_result = zeros(Rmax+1,K);
est_result(:,1) = [0:Rmax]'; % The first column is the number of the fitted factors
est_result(:,2:K+1) = beta_bias_corrected';
est_result

sd_result = zeros(Rmax+1,K);
sd_result(:,1) = [0:Rmax]'; % The first column is the number of the fitted factors
sd_result(:,2:K+1) = std_est_hetero';
sd_result

t_result = zeros(Rmax+1,K);
t_result(:,1) = [0:Rmax]'; % The first column is the number of the fitted factors
t_result(:,2:K+1) = beta_bias_corrected'./std_est_hetero';
t_result



outputfile='results_nolag.csv';
disp(['Write results into file: ' outputfile])  
  
%open output file
  fid = fopen(outputfile,'wt+');

%write parameters:
  fprintf(fid, 'Results from Empirical Application: with Lagged Dependent Variable \n\n');
  fprintf(fid, 'N, %d\n', N);
  fprintf(fid, 'T, %d\n', T);
  fprintf(fid, 'K, %d\n', K);
  fprintf(fid, 'Rmax, %d\n\n\n', Rmax);

%write header:
  fprintf(fid, ' ,');
  for R=0:Rmax
    fprintf(fid, ' R = %d,',R);
  end  
  fprintf(fid, '\n');
  
  
%write estimates:
  nameX=['years 1-2  '; 'years 3-4  '; 'years 5-6  '; 'years 7-8  '; 'years 9-10 '; 'years 11-12'; 'years 12-14'; 'years 15+  '];
  for k=1:K
     fprintf(fid, nameX(k,:));
     fprintf(fid, ', ');
     for R=0:Rmax
       b=beta_bias_corrected(k,1+R);
       s=std_est_hetero(k,1+R);
       t=b/s;
       fprintf(fid, '%.3f', b);
       if abs(t)>norminv(0.995)
         fprintf(fid, '** ,', b);
       elseif abs(t)>norminv(0.975)
         fprintf(fid, '*  ,', b);
       else
         fprintf(fid, '   ,', b);
       end
     end
     fprintf(fid, '\n ,');
     for R=0:Rmax
       b=beta_bias_corrected(k,1+R);
       s=std_est_hetero(k,1+R);
       t=b/s;
       fprintf(fid, '="(%.2f)",', t);
     end
     fprintf(fid, '\n');
  end   


fclose(fid);  

