clear all;
addpath('./..');    %add directory that contains data 
addpath('./../..'); %add directory that contains function "LS_factor" 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%  LOAD DATA (data and code for loading are from Kim and Oka):
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

DATA  = csvread('DivorceRate1956_1988.csv', 1, 1);
% In csvread(), 2nd and 3rd arguments are starting row and col to read.

% Data Definition

% State ID and Year
% Note: DATA is sorted by id_st and year in order.
id_st = DATA(:,1);

% Drop IN (id == 16) and NM (id == 33)
ind02  = (id_st ~= 16 & id_st ~= 33);
DATA02 = DATA(ind02,:);
id_st = DATA02(:,1);
year  = DATA02(:,2);
Y02 = (DATA02(:,4)); % Dependent variable%

% State population:
stpop = DATA02(:,5);

% Dummy for Friedberg's unilateral law
d_unilateral = DATA02(:,6);

% Dynamic Treatment effects
% - - - Friedberg's unilateral law
dyn_unilateral = DATA02(:,7:14);

% sample size
NT = size(DATA02, 1);
T  = max(year) - min(year) + 1;
N  = NT / T;

% dimension of the regressors
K = 8;   %will add +1 for lagged dependent variable below

%dependend variable and regressors in NT-vector form:

YY=Y02;
XX=dyn_unilateral;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%  TRANSFORM DATA INTO NxT MATRIX FORM:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Y = (reshape(YY,[T,N]))';
X = zeros(K,N,T);
for k=1:K
  X(k,:,:) = (reshape(XX(:,k),[T,N]))';
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%  ADD LAGGED DEPENDENT VARIABLE:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Ylag=Y(:,1:T-1);
Y=Y(:,2:T);
X=X(:,:,2:T);
X(2:K+1,:,:)=X;
X(1,:,:)=Ylag;  %FIRST regressor is lagged dependent variable

K=K+1;
T=T-1;

K
N
T

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%  CALCULATE RESIDUALS:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

beta0 = zeros(K,1);    
Rmax = 9 % the maximum number of the factors

lambda_known=ones(N,1);  % control for standard time effects
%f_known=ones(T,1);       % control for standard individual specific effects

trend=(1:T)';
f_known=[ones(T,1),trend,trend.^2]; %control for standard individual specific effects + time trend + time trend.^2


Rmax;
[beta,exitflag,lambda,f,Vbeta1,Vbeta2,Vbeta3,bcorr1,bcorr2,bcorr3]=LS_factor2(Y,X,Rmax,lambda_known,f_known,'silent',10^-8,'m1',beta0,30,300,2,0);

beta_est = beta - bcorr1 - bcorr2- bcorr3; %bias corrected estimator for beta

%calculate residuals Y-beta_est*X:

res=Y;
for k=1:size(X,1)
  res=res-beta_est(k)*squeeze(X(k,:,:));
end

M_lambda_known = eye(N) - lambda_known*inv(lambda_known'*lambda_known)*lambda_known';
M_f_known = eye(T) - f_known*inv(f_known'*f_known)*f_known';

res = M_lambda_known * res * M_f_known;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%  Calculate leading principal component (factor):
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

r=1
[V,D]=eig(res'*res);
[Dsort,Ind]=sort(diag(D));  
f=V(:,Ind(T-r+1:T));
%plot(f,'*')
%xlabel('time');
%ylabel('factor');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%  SCREE PLOT:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


ev = eig((res'*res)); %eigenvalues of res' * res (TxT or NxT matrix, depending what smaller)
ev = -sort(-ev);
ev = ev(1:T-size(f_known,2))

%figure;
%h=plot(ev,'b.');
%  axis([0 30 0 max(ev)+0.01]);
%  set(gca,'FontSize',20);
%  set(h,{'markers'},{20}) ;
%  xh=xlabel('factor number');
%  set(xh,'Fontsize',30);
%  yh=ylabel('eigenvalue');
%  set(yh,'Fontsize',30);
%figure;

h=plot(log(ev),'b.');
  axis([0 30 min(log(ev))-0.1 max(log(ev))+0.1]);
  set(gca,'FontSize',20);
  set(h,{'markers'},{20}) ;
  xh=xlabel('factor number');
  set(xh,'Fontsize',30);
  yh=ylabel('log(eigenvalue)');
  set(yh,'Fontsize',30);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%  ESTIMATE NUMBER OF FACTORS:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
  
%apply Bai-Ng and other criteria on residuals to estimate R:

if N<T
  res=res';  %make sure that the second dimension of res is the smaller one
end

ev = eig((res'*res)/(N*T)); %eigenvalues of res' * res (TxT or NxT matrix, depending what smaller)
ev = sort(ev);
ev_reverse = sort(ev,'descend');
SSRmax = sum(ev(1:min(N,T)-Rmax))

for R=0:Rmax
  SSR=sum(ev(1:min(N,T)-R)); %SSR after subtracting R leading factors (i.e. eigenvalues)
  SSR1=sum(ev(1:min(N,T)-(R+1)));
  SSR2=sum(ev(1:min(N,T)-(R+2)));
  
  IC1(R+1)=log(SSR)+R*(N+T)/N/T*log(N*T/(N+T));
  IC2(R+1)=log(SSR)+R*(N+T)/N/T*log(min(N,T));
  IC3(R+1)=log(SSR)+R*log(min(N,T))/min(N,T);
  
  PC1(R+1)=SSR+R*SSRmax*(N+T)/N/T*log(N*T/(N+T));
  PC2(R+1)=SSR+R*SSRmax*(N+T)/N/T*log(min(N,T));
  PC3(R+1)=SSR+R*SSRmax*log(min(N,T))/min(N,T);
  
  BIC3(R+1) = SSR + R*SSRmax*(N+T-R)*log(N*T)/(N*T);
  Diff(R+1) = ev_reverse(R+1) - ev_reverse(R+2);
  
  if R == 0
      mockev = SSR/log(min(N,T));
      Ratio(R+1) = mockev/ev_reverse(R+1);
      Gratio(R+1) = log(1+mockev/SSR)/log(1+ev_reverse(R+1)/SSR1)
  else
     Ratio(R+1) = ev_reverse(R)/ev_reverse(R+1);
     Gratio(R+1) = log(1+ev_reverse(R)/SSR)/log(1+ev_reverse(R+1)/SSR1);
  end
end



  [min_value, R_hatIC1] = min(IC1);
  [min_value, R_hatIC2] = min(IC2);
  [min_value, R_hatIC3] = min(IC3);

  R_estIC1=R_hatIC1-1
  R_estIC2=R_hatIC2-1
  R_estIC3=R_hatIC3-1
  
  [min_value, R_hatPC1] = min(PC1);
  [min_value, R_hatPC2] = min(PC2);
  [min_value, R_hatPC3] = min(PC3);

  R_estPC1=R_hatPC1-1
  R_estPC2=R_hatPC2-1
  R_estPC3=R_hatPC3-1
  
  [min_value, R_hatBIC3] = min(BIC3);
  R_estBI3=R_hatBIC3-1

  % [max_value, R_hatED] = max(Diff);
  % R_estED = R_hatED

  [max_value, R_hatER] = max(Ratio);
  R_estER = R_hatER-1
  
  [max_value, R_hatGR] = max(Gratio);
  R_estGR = R_hatGR-1
  
  
 
%change sorting of ev:

ev=ev(T:-1:1);
  
%calcualte Onatski criterion

j_old=inf;
j=Rmax+1;
r = 0;
while j~=j_old
  yyy=ev(j:j+4);
  xxx=[j-1:j+3]'.^(2/3);
  b=[ones(5,1),xxx]\yyy;
  delta=2*abs(b(2));
  if length( find( ev(1:T-1)-ev(2:T)>=delta ) ) ==0
    r=0; 
  else
    r=max(find( ev(1:T-1)-ev(2:T)>=delta ));
  end
  j_old=j;
  j=r+1;
end  

R_estED=r


% open output file 
  fout = fopen('factor_number_est_result_lagged.txt','wt+');
  fprintf(fout,'Estimation Results of the Number of Factors \n');
  fprintf(fout,'Maximum Number of Factors = %d, \n', Rmax);
  fprintf(fout,'IC1 of Bai and Ng (2004) = %d. \n', R_estIC1);
  fprintf(fout,'IC2 of Bai and Ng (2004) = %d. \n', R_estIC2);
  fprintf(fout,'IC3 of Bai and Ng (2004) = %d. \n', R_estIC3);
  fprintf(fout,'PC_p1 of Bai and Ng (2004) = %d. \n', R_estPC1);
  fprintf(fout,'PC_p2 of Bai and Ng (2004) = %d. \n', R_estPC2);
  fprintf(fout,'PC_p3 of Bai and Ng (2004) = %d. \n', R_estPC3);
  fprintf(fout,'BIC3 of Bai and Ng (2004) = %d. \n', R_estBI3);
  fprintf(fout,'ED of Onatski (2010) = %d. \n', R_estED);
  fprintf(fout,'ER of Ahn and Horenstein (2013) = %d. \n', R_estER);
  fprintf(fout,'GR of Ahn and Horenstein (2013) = %d. \n', R_estGR);
  fclose(fout);

  


