clear all;
addpath('./../..');    %add directory that contains function "LS_factor" 
addpath('./../../Empirical-Application'); %add directory that contains data 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%  LOAD DATA (data and code for loading are from Kim and Oka):
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

DATA  = csvread('DivorceRate1956_1988.csv', 1, 1);
% In csvread(), 2nd and 3rd arguments are starting row and col to read.

% Data Definition

% State ID and Year
% Note: DATA is sorted by id_st and year in order.
id_st = DATA(:,1);

% Drop IN (id == 16) and NM (id == 33)
ind02  = (id_st ~= 16 & id_st ~= 33);
DATA02 = DATA(ind02,:);
id_st = DATA02(:,1);
year  = DATA02(:,2);
Y02 = (DATA02(:,4)); % Dependent variable%

% State population:
stpop = DATA02(:,5);

% Dummy for Friedberg's unilateral law
d_unilateral = DATA02(:,6);

% Dynamic Treatment effects
% - - - Friedberg's unilateral law
dyn_unilateral = DATA02(:,7:14);

% sample size
NT = size(DATA02, 1);
T  = max(year) - min(year) + 1;
N  = NT / T;

% dimension of the regressors
K = 8;   %will add +1 for lagged dependent variable below

% dependend variable and regressors in NT-vector form:

YY=Y02;
XX=dyn_unilateral;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%  TRANSFORM DATA INTO NxT MATRIX FORM:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Y = (reshape(YY,[T,N]))';
X = zeros(K,N,T);
for k=1:K
  X(k,:,:) = (reshape(XX(:,k),[T,N]))';
end

K
N
T


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%  ESTIMATE PARAMETERS FOR R=4, STATIC MODEL, LS PROCEDURE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

R = 4 % the maximum number of the factors

lambda_known=ones(N,1);  % control for standard time effects

trend=(1:T)';
f_known=[ones(T,1),trend,trend.^2]; %control for standard individual specific effects + time trend + time trend.^2

[beta,exitflag,lambda,f,Vbeta1,Vbeta2,Vbeta3,bcorr1,bcorr2,bcorr3]=LS_factor2(Y,X,R,lambda_known,f_known,'silent',10^-8,'m1',zeros(K,1),30,300,2,2);

%bias corrected beta is set to be the true parameter:
beta0=beta - bcorr2 - bcorr3

%estimated lambda and f are set to be true parameters:
lambda0=lambda;
f0=f;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%  ESTIMATE REMAINING PARAMETERS:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%form residuals:
res=Y - lambda0*f0';
for k=1:K
  res=res-beta0(k)*squeeze(X(k,:,:));
end

Pf_known=f_known*inv(f_known'*f_known)*f_known';
Mf_known=eye(T)-Pf_known;
Plambda_known=lambda_known*inv(lambda_known'*lambda_known)*lambda_known';
Mlambda_known=eye(N)-Plambda_known;

res1=Mlambda_known*res*Mf_known;

%get estimates for alpha, gamma, delta, set to be true parameters for DGP:
alpha_gamma_delta0=Mlambda_known*res*f_known*inv(f_known'*f_known);
res = res - alpha_gamma_delta0 * f_known';

%get estimates for mu, set to be true parameters for DGP:
mu0=( inv(lambda_known'*lambda_known)*lambda_known'*res )';
res = res - lambda_known*mu0';

% % %estimate an AR(1) model for the residuals:
% % 
% % res1=res(:,2:T);
% % resLag=res(:,1:T-1);
% % rho_est = trace(res1*resLag') / trace(resLag*resLag')
% % 
% % % set AR(1) coefficient
% % 
% % rho=0.5
% % resAR=res1-rho*resLag;
% % sigma = sqrt( trace(resAR*resAR') /(N-R-1) /(T-R-3-1) )
% % 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%  MONTE CARLO SIMULATION:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%fixed part of Y, which does not change during the simulation:
Yfixed=zeros(N,T);
for k=1:K
  Yfixed = Yfixed + beta0(k)*squeeze(X(k,:,:));
end
Yfixed = Yfixed + alpha_gamma_delta0 * f_known' + lambda_known*mu0' + lambda0*f0';

R0=4;
Rmax=9;
rep=10000;

for r=1:rep
  if mod(r,min(50,ceil(rep/20)))==0
    disp(['    ' num2str(r) ' / ' num2str(rep)]);
  end
  
  %DGP FOR ERROR TERM:
 
  dof=5;      %degrees of freedom for t-distribution of errors
  v = tinv(rand(N,T+1),dof);   %shocks are t-distributed with degrees of freedom = dof
  v0 = v(:,2:T+1); 
  v1 = v(:,1:T); 
  e = 0.1 * (v0 + v1);   %MA1 process for errors

  %GENERATE OUTCOME VARIABLES:
    
  Y = Yfixed + e;

  %set bandwidth parameters:
  
  M1=2;
  M2=2;
  
  %calculate OLS estimator:
  
   yy=Mlambda_known*Y*Mf_known;
   YY=yy(:);                      %flatten Y, i.e. YY now NTx1 vector
   for k=1:K
     xx=squeeze(X(k,:,:));        
     xx=Mlambda_known*xx*Mf_known;
     XX(:,k)=xx(:);               %flatten X, i.e. XX becomes NTxK matrix
   end    
   beta_OLS=pinv(XX'*XX)*XX'*YY;      %calculate OLS estimator
   res=YY-XX*beta_OLS;  
   
  %calculate variance estimators for OLS estimator: 
   
   XXinv=inv(XX'*XX);
   Omega1 = var(res(:)) * XXinv;
   Omega2 = XXinv * inv(XX'*diag(res.^2)*XX) * XXinv;
   for k1=1:K
   for k2=1:K
      WW(k1,k2)=sum(sum(trunc((res.*XX(:,k1))'*(res.*XX(:,k2)),M2+1,M2+1)));
   end
   end
   Omega3 = XXinv * WW * XXinv;
   
  for k=1:K
    beta_est(k,r,1)=beta_OLS(k);
    beta_BC1(k,r,1)=beta_OLS(k); %no bias correction
    beta_BC2(k,r,1)=NaN;         %formula for lagged dependent bias not implemented, yet
    est_std1(k,r,1)=sqrt( Omega1(k,k) );
    est_std2(k,r,1)=sqrt( Omega2(k,k) );
    est_std3(k,r,1)=sqrt( Omega3(k,k) );
  end
    
  %calculate QMLE with different R:
  
  for R=1:Rmax
    start_beta=beta_OLS;
    repMIN=3;
    repMAX=10;
    [beta,exitflag,lambda,f,Vbeta1,Vbeta2,Vbeta3,bcorr1,bcorr2,bcorr3]=LS_factor2(Y,X,R,lambda_known,f_known,'silent',10^-8,'m1',start_beta,repMIN,repMAX,M1,M2);
    if exitflag<=0
      disp('QMLE OPTIMIZATION PROBLEM, may need to increase repMIN or repMAX or optimset-parameters in function qmle')  
    end 
    for k=1:K
      beta_est(k,r,1+R)=beta(k);        %LS-estimator using R factors in estimation
      beta_BC1(k,r,1+R)=beta(k)-bcorr2(k)-bcorr3(k); %bias corrected estimator (only using static biases)
      beta_BC2(k,r,1+R)=beta(k)-bcorr1(k)-bcorr2(k)-bcorr3(k); %bias corrected estimator (using all bias terms)
      est_std1(k,r,1+R)=sqrt( Vbeta1(k,k) ); %estimated standard error, assuming homoscedasticity
      est_std2(k,r,1+R)=sqrt( Vbeta2(k,k) ); %estimated standard error, assuming heteroscedasticity
      est_std3(k,r,1+R)=sqrt( Vbeta3(k,k) ); %estimated standard error, allowing time-serial correlation
    end
  end

end  

save('results.mat')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%  REPORT RESULTS:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

outputfile='MC_Empirical_Results.csv';

%open output file
  fid = fopen(outputfile,'wt+');

%write parameters:
  fprintf(fid, 'rep, %d\n', rep);
  fprintf(fid, 'N, %d\n', N);
  fprintf(fid, 'T, %d\n', T);
  fprintf(fid, 'R0, %d\n', R0);
  fprintf(fid, 'Rmax, %d\n', Rmax);
  fprintf(fid, '\n\n');

  fprintf(fid, 'LS-factor: uncorrected estimator: \n\n');
    
  fprintf(fid, '      ,');
  for k=1:K
    fprintf(fid, ', k = %d , , ',k);
  end
  fprintf(fid, '\n');
  fprintf(fid, '      ,');
  for k=1:K
    fprintf(fid, 'BIAS, SD, SIZE, ',R);
  end
  fprintf(fid, '\n');
  for R=0:Rmax
    fprintf(fid, 'R = %d,   ',R);
    for k=1:K  
      b=squeeze(beta_est(k,:,R+1));
      bias_est=mean(b)-beta0(k);
      std_est=sqrt(mean((b-mean(b)).^2));
      rmse_est=sqrt(mean((b-beta0(k)).^2));
      size=sum( abs( (b-beta0(k))./est_std3(k,:,1+R) ) > norminv(0.975) )/rep;
      fprintf(fid, '%6.4f ,%6.4f, %6.4f, ',bias_est,std_est,size);
    end
    fprintf(fid,'\n');
  end

  fprintf(fid, '\n\n');
  fprintf(fid, 'LS-factor: bias corrected estimator: \n\n');

  fprintf(fid, '      ,');
  for k=1:K
    fprintf(fid, ', k = %d , , ',k);
  end
  fprintf(fid, '\n');
  fprintf(fid, '      ,');
  for k=1:K
    fprintf(fid, 'BIAS, SD, SIZE, ',R);
  end
  fprintf(fid, '\n');
  for R=0:Rmax
    fprintf(fid, 'R = %d,   ',R);
    for k=1:K  
      b=squeeze(beta_BC1(k,:,R+1));
      bias_est=mean(b)-beta0(k);
      std_est=sqrt(mean((b-mean(b)).^2));
      rmse_est=sqrt(mean((b-beta0(k)).^2));
      size=sum( abs( (b-beta0(k))./est_std3(k,:,1+R) ) > norminv(0.975) )/rep;
      fprintf(fid, '%6.4f ,%6.4f, %6.4f, ',bias_est,std_est,size);
    end
    fprintf(fid,'\n');
  end
  
    

fclose(fid);  
  












